"""
Package that enables Django to easily and quickly be configured to pass
X-Sendfile or X-Accel-Redirect requests to web servers for files that need to
have some sort of permimssions check run prior to serving.
"""

from .box import GiftBox

from .version import __version__, version_info



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from .wrappers import send_dev_server, xsendfile


class GiftBox(object):
    """Container class for sendfile wrappers

    Attributes:
        request (HttpRequest): Instance of :class:`django.http.HttpRequest`
        wrapper (function): Instance of either
            :func:`~giftbox.wrappers.send_dev_server` or
            :func:`~giftbox.wrappers.xsendfile`
        kwargs (dict): A dictionary containing kwargs passed to the object,
            used by the wrappers to accept settings whether
            from ``settings.py`` or :class:`GiftBox` object on creation.
    """

    def __init__(self, request, **kwargs):
        """
        Create a :class:`GiftBox` instance.

        Args:
            request (HttpRequest): Instance of
            :class:`django.http.HttpRequest`.

        Keyword Args:
            doc_root (str): Valid filepath for Django's development server
                                to 'xsend' files.
        """

        # Object has access to ``self.request``, ``self.kwargs``, and
        # ``self.wrapper``.
        self.request = request
        self.wrapper = None
        self.kwargs = dict()

        # Check for development server running
        server = request.META.get('SERVER_SOFTWARE', None)
        if server is not None and \
                'WSGIServer' in server:
            self.wrapper = send_dev_server

        # Check for GIFTBOX_SETTINGS
        gbs = getattr(settings, 'GIFTBOX_SETTINGS', None)
        if not gbs:
            raise ImproperlyConfigured('Please configure GIFTBOX_SETTINGS.')

        # Use settings to determine wrapper, if not running development server.
        if 'type' in gbs and not self.wrapper:
            if gbs['type'] == 'dev':
                self.wrapper = send_dev_server
            elif gbs['type'] == 'prod':
                self.wrapper = xsendfile

        self.kwargs['doc_root'] = gbs.get('doc_root', None)
        self.kwargs['use_magic'] = gbs.get('use_magic', None)

        try:
            import magic
            self.kwargs['has_magic'] = True
            # set if unspecified since magic has been detected
            if self.kwargs['use_magic'] is None:
                self.kwargs['use_magic'] = True
        except ImportError:
            if self.kwargs['use_magic']:
                raise ImproperlyConfigured('To enable magic for mime-typing, install python-magic and libmagic.')
            self.kwargs['has_magic'] = False

        self.kwargs.update(kwargs)

        if not self.kwargs['doc_root']:
            raise ImproperlyConfigured('You must specify a "doc_root"')

    def send(self, filename, **kwargs):
        """
        Return an HTTP Response to send the specified file.

        Args:

            filename (str): The name of a file to serve.
            doc_root (str): Valid path for Django's server to 'xsend'
            use_magic (bool): whether or not to pythonmagic
        """
        obj_kwargs = self.kwargs.copy()
        obj_kwargs.update(kwargs)
        # If somehow a wrapper hasn't been set yet.
        if not self.wrapper:
            raise ImproperlyConfigured('You must specify a wrapper before '
                                       'using send.')
        return self.wrapper(self.request, filename, **obj_kwargs)



import os
from django.views.static import serve
from django.http import HttpResponse


def get_mime(filepath):
    """
    Use python-magic to get the mime type of a file.

    Args:
        filepath (str): Path to the file to be sniffed by magic

    Returns:
        str: Returns a string representing the mime type of the file.
    """
    import magic
    return magic.from_file(filepath, mime=True)

        
def send_dev_server(request, filename, **kwargs):
    """
    Send a file using Django's development server.

    Args:
        request (HttpRequest): An instance of :class:`django.http.HttpRequest`
        filename (str): name of the file to be served

    Returns:
        FileResponse: An instance of class:`django.http.FileResponse`.

    """
    response = serve(request, filename, kwargs['doc_root'])
    # Django tries to pick an intelligent mime type
    # If magic, use it to help out
    if kwargs['use_magic'] and kwargs['has_magic']:
        response['Content-Type'] = get_mime(
           os.path.join(kwargs['doc_root'], filename)
        )
    response['Content-Disposition'] = ('attachment; filename=%s'
                                       % filename.split('/')[-1])
    return response


def xsendfile(request, filename, **kwargs):
    """
    Send a file using an HTTP X-Sendfile.

    Args:
        request (HttpRequest): An instance of :class:`django.http.HttpRequest`
        filename (str): Name of the file to be served

    Returns:
        HttpResponse: An instance of :class:`django.http.HttpResponse`.
 
    """

    response = HttpResponse()
    path = os.path.join(kwargs['doc_root'], filename)
    response['X-Sendfile'] = path
    response['Content-Disposition'] = 'attachment; filename=%s' % filename
    # Delete default 'Content-Type', which indicates HTML, and let web server
    # try to get it right.
    del response['Content-Type']
    # If magic available and not explicitly disabled, use it to help out
    if kwargs['use_magic'] and kwargs['has_magic']: 
        response['Content-Type'] = get_mime(os.path.join(kwargs['doc_root'], 
                                            filename))
    return response



"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Credit to Kenneth Reitz's remarkably sane
# https://github.com/kennethreitz/setup.py

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path


here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

about = {}
with open(path.join(here, 'giftbox', 'version.py')) as f:
    exec(f.read(), about)


TESTS_REQUIRE = ['pytest', 'pytest-django', 'pytest-cov',
                 'mock', 'python-magic', 'tox', 'tox-travis']


setup(
    name='django-giftbox',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=about['__version__'],

    description='A package that includes xsendfile capabilities for Django',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/bwhicks/django-giftbox',

    # Author details
    author='Benjamin Hicks',
    author_email='benajmin.w.hicks@gmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 1.8',
    ],

    # What does your project relate to?
    keywords='sendfile, apache',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    setup_requires=['pytest-runner'],
    tests_require=TESTS_REQUIRE,
    install_requires=['django>=1.8'],
    extras_require={
        'magic': ['python-magic'],
        'test': TESTS_REQUIRE,
        'develop': TESTS_REQUIRE + ['twine']
    },

)
